$title Standard QP Model (QP1,SEQ=171): Modified for risk/return balance

$offlisting
$Ontext
 The first in a series of variations on the standard
 QP formulation. The subsequent models exploit data
 and problem structures to arrive at formulations that
 have sensational computational advantages. Additional
 information can be found at:

 http://www.gams.com/modlib/adddocs/qp1doc.htm

Brooke, A, Kendrick, D, and Meeraus, A, GAMS: A User's Guide. The
Scientific Press, Redwood City, California, 1988.

This version is modified to make the objective function a tradeoff
between risk and return, where return is quantified by the mean of the
portfolio and risk by its variance. A nonnegative parameter rho is
used to weight these two objective. Smaller values of rho place more
weight on reducing risk; larger values place more weight on increasing
return.

Good values to try are rho=0.1, 1.0, 10.

$Offtext
$onlisting

option limrow=0, limcol=0, solprint=off;

$include qpdata.inc

* rho values 10, 1, .5
scalar rho rho rho your boat,
       maxStocks     max. amount of stocks handled /8/,
       maxShort      max. amount of stocks to short sell /1/,
       minPctBudget  min. percent of budget to invest per stock bought /.10/,
       maxShortPct   max. percent of budget to invest per stock shorted /.20/,
       lilM          not B.I.G. but lil'. unused. just chillin'.;
scalar meanFinal, varFinal;

set d(days)   selected days
    s(stocks) selected stocks
    i number of rhos /1*3/;

alias(s,t);

set properties /avReturn, variance/;

* setect subset of stocks and periods
d(days)   = ord(days) >= 1 and ord(days) <= 90;
s(stocks) = ord(stocks) <= 100;

parameter rhoX(i) /1 10, 2 1, 3 .5/,
          mean(stocks)          mean of daily return
          dev(stocks,days)      deviations
          covar(stocks,sstocks) covariance matrix of returns (upper)
          meanvar(stocks,properties)  mean and var of each stock
          totmean               total mean return;

mean(s)  = sum(d, return(s,d))/card(d);
dev(s,d) = return(s,d)-mean(s);

* calculate covariance
* to save memory and time we only compute the uppertriangular
* part as the covariance matrix is symmetric
covar(upper(s,t)) = sum(d, dev(s,d)*dev(t,d))/(card(d)-1);

* fill out the properties of each stock and print
meanvar(stocks,'avReturn') = mean(stocks);
meanvar(stocks,'variance') = covar(stocks,stocks);

display meanvar;

totmean = sum(s, mean(s))/(card(s));

free variables z          objective,
               x(stocks)  investments;

binary variables
          buy(stocks)    indicates if stock is bought,
          short(stocks)  indicates if stock is shorted;

equations EQ_obj    objective,
          EQ_budget,
          EQ_sumHandled,
          EQ_sumShort,
          EQ_maxPctBudget(stocks),
          EQ_minPctBudget(stocks);

* objective function
EQ_obj..
         z =e= rho * sum(s, mean(s)*x(s)) -
               0.5 * (sum(upper(s,t), x(s)*covar(s,t)*x(t)) +
                      sum(lower(s,t), x(s)*covar(t,s)*x(t)));

* stay within budget
EQ_budget..
         sum(s, x(s)) =e= 1.0;

* limit number of stocks that can be handled
EQ_sumHandled..
         sum(s, buy(s) + short(s)) =l= maxStocks;

* limit number of stocks that can be shorted
EQ_sumShort..
         sum(s, short(s)) =l= maxShort;

* push unbought stocks upper bound to 0
EQ_maxPctBudget(s)..
         x(s) =l= buy(s);

* enforce lower bound of amount of stock bought/shorted
EQ_minPctBudget(s)..
         x(s) =g= minPctBudget * buy(s) - maxShortPct * short(s);

model qp1 /all/;
option minlp = sbb;
option iterlim = 100000;
qp1.nodlim = 100000;

file portopt/portopt.txt/;
put portopt;
put 'Portfolio'/;

loop(i,
*        reset values before each solve
         rho = rhoX(i);
         x.l(s) = 0;
         buy.l(s) = 0;
         short.l(s) = 0;

         solve qp1 using minlp maximizing z;

*        remove rounding errors (near zero numbers)
         loop(s,
                 if(abs(x.l(s)) < 10**(-10),
                         x.l(s) = 0;
                 );
         );

*        print to file
         put /'rho: ' rho/
         put /'Investments'/
         loop(s$(x.l(s) <> 0),
                 put @3, s.tl, @15, x.l(s):8:4/;
         );

*        calculate values
         meanFinal = sum(s, mean(s)*x.l(s));
         varFinal = sum(upper(s,t), x.l(s)*covar(s,t)*x.l(t)) +
                    sum(lower(s,t), x.l(s)*covar(t,s)*x.l(t));

         put /'Objective:', @15, z.l:8:4/;
         put 'Mean return:', @15, meanFinal:8:4/;
         put 'Variance:', @15, varFinal:8:4/;

);

putclose portopt;
