$title Standard QP Model (QP1,SEQ=171): Modified for risk/return balance

$offlisting
$Ontext
 The first in a series of variations on the standard
 QP formulation. The subsequent models exploit data
 and problem structures to arrive at formulations that
 have sensational computational advantages. Additional
 information can be found at:

 http://www.gams.com/modlib/adddocs/qp1doc.htm

Brooke, A, Kendrick, D, and Meeraus, A, GAMS: A User's Guide. The
Scientific Press, Redwood City, California, 1988.

This version is modified to make the objective function a tradeoff
between risk and return, where return is quantified by the mean of the
portfolio and risk by its variance. A nonnegative parameter rho is
used to weight these two objective. Smaller values of rho place more
weight on reducing risk; larger values place more weight on increasing
return.

Good values to try are rho=0.1, 1.0, 10.

$Offtext
$onlisting

*option limrow=0, limcol=0;

$include qpdata.inc

scalar rho 10 1 .5 /.1/,
       maxStocks     max. amount of stocks handled /8/,
       maxShort      max. amount of stocks to short sell /1/,
       minPctBudget  min. percent of budget to invest per stock bought /.10/,
       maxShortPct   max. percent of budget to invest per stock shorted /.20/;

set d(days)   selected days
    s(stocks) selected stocks
alias(s,t);

set properties /avReturn, variance/;

* setect subset of stocks and periods
d(days)   = ord(days) >= 1 and ord(days) <= 90;
s(stocks) = ord(stocks) <= 100;

parameter mean(stocks)          mean of daily return
          dev(stocks,days)      deviations
          covar(stocks,sstocks) covariance matrix of returns (upper)
          meanvar(stocks,properties)  mean and var of each stock
          totmean               total mean return;

mean(s)  = sum(d, return(s,d))/card(d);
dev(s,d) = return(s,d)-mean(s);

* calculate covariance
* to save memory and time we only compute the uppertriangular
* part as the covariance matrix is symmetric
covar(upper(s,t)) = sum(d, dev(s,d)*dev(t,d))/(card(d)-1);

* fill out the properties of each stock and print
meanvar(stocks,'avReturn') = mean(stocks);
meanvar(stocks,'variance') = covar(stocks,stocks);

display meanvar;

totmean = sum(s, mean(s))/(card(s));

free variables z          objective;

positive variables
          x(stocks)  investments;

binary variables
          active(stocks) indicates if stock is handled,
          short(stocks)  indicates if stock is shorted;

equations EQ_obj    objective,
          EQ_budget,
*$ontext
          EQ_sumActive,
          EQ_sumShort,
          EQ_activeShort(stocks),
          EQ_maxPctBudget(stocks),
          EQ_minPctBudget(stocks),
          EQ_maxShortPct(stocks);
*$offtext

* objective function
EQ_obj..
         z =e= rho * sum(s, mean(s)*x(s)) -
               0.5 * (sum(upper(s,t), x(s)*covar(s,t)*x(t)) +
                      sum(lower(s,t), x(s)*covar(t,s)*x(t)));

* stay within budget
EQ_budget..
         sum(s, x(s)) =e= 1.0;

*$ontext
* limit number of stocks that can be handled
EQ_sumActive..
         sum(s, active(s)) =l= maxStocks;

* limit number of stocks that can be shorted
EQ_sumShort..
         sum(s, short(s)) =l= maxShort;

EQ_activeShort(s)..
         active(s) =g= short(s);

EQ_maxPctBudget(s)..
         x(s) =l= active(s) + short(s);

* enforce lower bound of amount of stock bought
EQ_minPctBudget(s)..
         x(s) =g= minPctBudget * active(s);

* enforce upper bound on shorted stocks as percent of budget
EQ_maxShortPct(s)..
         x(s) =l= maxShortPct * short(s) + (1 - short(s));
*$offtext

model qp1 /all/;

solve qp1 using minlp maximizing z;

display x.l;

scalar meanFinal, varFinal;
meanFinal = sum(s, mean(s)*x.l(s));
varFinal = sum(upper(s,t), x.l(s)*covar(s,t)*x.l(t)) +
            sum(lower(s,t), x.l(s)*covar(t,s)*x.l(t));
display meanFinal, varFinal;
